#ifdef ARDUINO_RASPBERRY_PI_PICO_W
#include <WiFi.h>
#include <WebServer.h>
#include <WebSocketsServer.h> // arduinoWebSockets library
#include <LEAmDNS.h>

WebServer server(80);
WebSocketsServer webSocket = WebSocketsServer(81);

// WiFi information
//#define WIFI_ACCESS_POINT
#ifndef WIFI_ACCESS_POINT
const char* ssid = "XXXX";
const char* pass = "YYYY";
#else
const char* apssid = "RPPICOFFT";
const char* password = "12345678";
const IPAddress ip(192, 168, 4, 1);
const IPAddress subnet(255, 255, 255, 0);
#endif

// Initial Screen
void handleRoot(void) {
  if (server.method() == HTTP_POST) {
    Serial.println(server.argName(0));
    handle_range();
    return;
  }
  index_html(NULL);
}

void handle_range() {
  String val = server.arg("range");
  if (val != NULL) {
    Serial.println(val);
    if (val == "1") {                     // fast
      if (range < 13) wrange = 13;
    } else if (val == "0") {              // slow
      if (range > 3) wrange = 3; 
    }
    Serial.println(wrange);
    server.send(200, "text/html", "OK");  // response 200, send OK
  }
}

void index_html(void * pvParameters) {
String html;

//<meta http-equiv="refresh" content="1; URL=">
  html = R"=====(
<!DOCTYPE html>
<html>
<head><meta charset="utf-8"><title>FFT Analyzer</title>
<style>body { background: #fafafa; }
canvas { display:block; background:#ffe; margin:0 auto; }</style>
<script type="text/javascript">
var ws = new WebSocket('ws://' + window.location.hostname + ':81/');
ws.binaryType = 'arraybuffer';
ws.onmessage = function(evt) {
  var wsRecvMsg = evt.data;
  var datas = new Int16Array(wsRecvMsg);
  var cv1 = document.getElementById('cvs1');
  var ctx = cv1.getContext('2d');
  if (!cv1||!cv1.getContext) { return false; }
  var groundW= cv1.getAttribute('width');
  var groundH= cv1.getAttribute('height');
  ctx.fillStyle = "rgb(0,0,0)";
  ctx.fillRect(0,0,groundW,groundH);
  var groundX0= 0; var groundY0= groundH;
  var pichX = 40;
  var cnstH= groundH/4096;
  var pichH = groundH/8;
  const fftsamples = 128;
  const displng = 256;
  ctx.beginPath();
  ctx.save();
  ctx.strokeStyle = "rgb(128,128,128)";
  ctx.lineWidth = 1;
  for (var i = 0; i < 2*displng; i+=pichX){
    ctx.moveTo(groundX0+i, groundY0-pichH);
    ctx.lineTo(groundX0+i, 0);
    for (var j=8; j<40; j=j+8){
      ctx.moveTo(groundX0+i+j, 4*pichH-5);
      ctx.lineTo(groundX0+i+j, 4*pichH+5);
    }
  }
  for (var i = pichH; i < groundH; i+=pichH){
    ctx.moveTo(groundX0, groundY0-i);
    ctx.lineTo(2*displng, groundY0-i);
    for (var j=10; j<50; j=j+10){
      ctx.moveTo(6*pichX-5, groundY0-i-j);
      ctx.lineTo(6*pichX+5, groundY0-i-j);
    }
  }
  ctx.stroke();
  ctx.restore();
  ctx.save();
  ctx.beginPath();
  ctx.strokeStyle = "rgb(0,255,0)";
  if (datas.length > fftsamples) {
    var base = groundY0-cnstH*512;
    for (var i = 1; i < fftsamples; i++){
      ctx.moveTo(groundX0+4*i, base);
      ctx.lineTo(groundX0+4*i, base-datas[i]);
    }
    ctx.stroke();
    ctx.strokeStyle = "rgb(192,192,192)";
    ctx.moveTo(0, base); ctx.lineTo(0, base+8);
    ctx.moveTo(8*25, base); ctx.lineTo(8*25, base+8);
    ctx.moveTo(8*50, base); ctx.lineTo(8*50, base+8);
    ctx.textAlign = "left";
    ctx.textBaseline = "bottom";
    ctx.font = "12pt Arial";
    ctx.fillStyle = "rgb(192,192,192)";
    for (var j = 0; j < 70; j += 10){
      ctx.fillText(String(-j)+"dB", 525, 5 * j + 60);
    }
    var nyquist = 1000 * datas[fftsamples] + datas[fftsamples+1];
    var usec = Math.round(100000000 / nyquist);
    ctx.textBaseline = "top";
    ctx.fillText("0Hz", 0, base + 10);
    ctx.textAlign = "center";
    if ((nyquist/2) < 1000)
      ctx.fillText(String(nyquist/2)+"Hz", 8*25, base + 10);
    else
      ctx.fillText(String(nyquist/2000)+"kHz", 8*25, base + 10);
    if (nyquist < 1000)
      ctx.fillText(String(nyquist)+"Hz", 8*50, base + 10);
    else
      ctx.fillText(String(nyquist/1000)+"kHz", 8*50, base + 10);
    if (usec < 1000)
      ctx.fillText(String(usec)+"us", 530, 10);
    else if (usec < 1000000)
      ctx.fillText(String(usec/1000)+"ms", 530, 10);
    else
      ctx.fillText(String(usec/1000000)+"s", 530, 10);
  }
  if (datas.length >= displng) {
    groundY0 = 100;
    ctx.strokeStyle = "rgb(255,255,255)";
    ctx.moveTo(groundX0, groundY0-(cnstH*datas[fftsamples+2])/4);
    for (var i = 1; i < displng; i++){
      ctx.lineTo(groundX0+2*i, groundY0-(cnstH*datas[i+fftsamples+2])/4);
    }
  }
  ctx.stroke();
  ctx.restore();
};
</script>
</head>
<script type="text/javascript">
async function postrange(i) {
  try {
    var form = new FormData()
    form.append('range', i)
    const response = await fetch("/", {
      method: "POST",
      body: form
    });
    if (response.ok) {
      const text = await response.text();
      console.log(text);
    }
  } catch (error) { console.log(error); }
}
</script>
<body>
<h3>FFT Analizer v1.0</h3>
<div style='float: left; margin-right: 10px'>
<canvas id='cvs1' width='601' height='401' class='float'></canvas></div>
<form id='range0'>range: 
  <button type="button" name='range' value='1' onclick='postrange(this.value)'>FAST</button>
  <button type="button" name='range' value='0' onclick='postrange(this.value)'>SLOW</button></form>
</div>

</body>
</html>
)=====";

// send the HTML
  server.send(200, "text/html", html);
}

void handleNotFound(void) {
  server.send(404, "text/plain", "Not Found.");
}

void setup1() {
#ifdef WIFI_ACCESS_POINT
  WiFi.disconnect(true);
  delay(1000);
  WiFi.softAP(apssid, password);
  delay(100);
  WiFi.softAPConfig(ip, ip, subnet);
  IPAddress myIP = WiFi.softAPIP();
#else
// Connect to the WiFi access point
  WiFi.begin(ssid, pass);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
#endif
  // print out the IP address of the Server
  Serial.print("WiFi Connected. IP = "); Serial.println(WiFi.localIP());
//  Serial.print("WiFi Connected. GW = "); Serial.println(WiFi.gatewayIP());
//  Serial.print("WiFi Connected. DNS = "); Serial.println(WiFi.dnsIP());

  if (MDNS.begin("picowfft")) {
    Serial.println("MDNS responder started");
  }

  server.on("/", handleRoot);
  server.onNotFound(handleNotFound);
  server.begin();
  webSocket.begin();
}

void loop1() {
  server.handleClient();
  MDNS.update();
  if (rp2040.fifo.available() > 0) {
    sem_acquire_blocking(&sem);
    webSocket.broadcastBIN((byte *) payload, NNN + 4 + 2*NNN);
//    webSocket.broadcastBIN((byte *) payload, NNN + 4);          // without wave
    sem_release(&sem);
    while (rp2040.fifo.available() > 0) {
      rp2040.fifo.pop();
    }
  }
  webSocket.loop();
  delay(1);
}
#endif
